////////////////////////////////////////////////////////////////////////
////                          Lcd.c			                        ////
////                                                                ////
////  Fonction grant l'cran LCD			 						////
////                 18F252 quartz 10Mhz *4 (PLL) = 40Mhz 			////
////  Auteur: 	LG					     							////
////  Date:	 15 novembre 2003                                     	////
////                                                                ////
////  change by: 	Julien			     							////
////  Date:	 7 mars 2004	                                     	////
////                                                                ////
////////////////////////////////////////////////////////////////////////
////        (C) Copyright 2004 - Fribotte		             		////
////								     							////
////////////////////////////////////////////////////////////////////////
#include "lcd.h"
#include <p18f252.h>

// all var in gpr1 bank (bank 0 )
#pragma udata FAR_s


void OS_Lcd_Send_noWait(uchar LUC_VarSend);				// envoi de commande dans delais
void OS_Lcd_Send(uchar LUC_VarSend);					// envoi de commande avec delais de 50us, minimum pour les ecrans


// 20x4 module
   //  Line 1 addresses are 00h to 13h
   //  Line 2 addresses are 40h to 53h
   //  Line 3 addresses are 14h to 27h
   //  Line 4 addresses are 54h to 67h

// 16*2 module
	// Ligne 1 addresses de 00h  0Fh
	// Ligne 2 addresses de 40h	 4Fh

// Pour information le cablage suivant pour un fonctionnement en mode 4 bits
// RB0 D0
// RB1 D1
// RB2 D2
// RB3 D3
// RB4 E
// RB5 RS
// R/W est forc  la masse, donc on ne peut pas lire sur l'cran.

// n'oubliez pas de rien regler le TRISB et TRISC
#define LCD_D7 PORTBbits.RB5
#define LCD_D6 PORTBbits.RB6
#define LCD_D5 PORTBbits.RB4
#define LCD_D4 PORTBbits.RB7

#define LCD_E  PORTBbits.RB3
#define LCD_RW PORTCbits.RC6
#define LCD_RS PORTBbits.RB2

//15ms 15 *10000 * 100ns
#define DELAY_15MS Delay10KTCYx(15);

//2ms 2 *10000 * 100ns
#define DELAY_2MS Delay10KTCYx(2);

// 600ns, 6 NOP
#define DELAY_600NS Delay1TCY();Delay1TCY();Delay1TCY();Delay1TCY();Delay1TCY();Delay1TCY();

// 40s 4 * 100 * 100ns
#define DELAY_40US Delay100TCYx(4);				

// 60s 6 * 100 * 100ns
#define DELAY_60US Delay100TCYx(6);



/********************************************************
Fonction: OS_IniLcd
Utilisation: Fonction d'initialisation de l'afficheur LCD
Date: 23/11/2003
Auteur: LG
********************************************************/

void OS_InitLcd(void)
{
	// dclaration de variables locales
	uchar LUC_VarSend;

	LCD_E  = 0; 
	LCD_RW = 0;
	LCD_RS = 0;

	// dbut du corps de la fonction 
	// boucle de tempo de 15ms nessaire au boot!
	DELAY_15MS;

	
	// System Set
	LUC_VarSend= 0x03; 							// Mode 8 bit 
	OS_Lcd_Send(LUC_VarSend);						// poids fort
	OS_Lcd_Send(LUC_VarSend);						// poids fort
	OS_Lcd_Send(LUC_VarSend);						// poids fort

	LUC_VarSend= 0x02;							// mode 4 bit
	OS_Lcd_Send(LUC_VarSend);						// poids fort

	LUC_VarSend= 0x02;							// mode 4 bit
	OS_Lcd_Send(LUC_VarSend);						// poids fort commande
	LUC_VarSend= 0x08;							// mode 4 bit
	OS_Lcd_Send(LUC_VarSend);						// poids faible commande

	// Set ON/OFF
	LUC_VarSend= 0x00; 							// display On, curseur On, Blink On
	OS_Lcd_Send(LUC_VarSend);
	LUC_VarSend = 0x0E;							// poids faible ecran On, Cursor On, Blink OFF
	OS_Lcd_Send(LUC_VarSend);	
							
	DELAY_2MS;	// delais de 2ms
	OS_Lcd_Clear();

	// Set entry mode
	LUC_VarSend = 0x00;
	OS_Lcd_Send(LUC_VarSend);
	LUC_VarSend = 0x06;							// poids faible
	OS_Lcd_Send(LUC_VarSend);
	OS_Lcd_Clear();
	DELAY_2MS;	// delais de 2ms
}

/********************************************************
Fonction: OS_Lcd_Ligne
Utilisation: Fonction de selection de ligne
Date: 23/11/2003
Auteur: LG
********************************************************/
void OS_Lcd_Line( uchar LUC_Ligne)
{
	// dclaration de variables locales
	
	// dbut du corps de la fonction 
	
// positionnement sur la ligne X.
/* cran 4 ligne 20 characteres
	if (LUC_Ligne == 1) LUC_Ligne= 0x80;
	if (LUC_Ligne == 2) LUC_Ligne =0xC0;							// poids fort ligne 2
	if (LUC_Ligne == 3) LUC_Ligne =0x90;							// poids fort ligne 3
	if (LUC_Ligne == 4) LUC_Ligne=0xD0;							// poids fort ligne 4
*/
// cran 2 lignes 16 characteres:
	if (LUC_Ligne == 1) LUC_Ligne= 0x08;							// poids fort ligne 1
	if (LUC_Ligne == 2) LUC_Ligne =0x0C;							// poids fort ligne 2
	
	OS_Lcd_Send(LUC_Ligne);
/* cran 4 lignes 
	if ((LUC_Ligne ==0x80) ||(LUC_Ligne == 0xC0))
		LUC_Ligne=0x00;							// poids faible ligne 1 et 2
	else
		LUC_Ligne = 0x40;				// poids faible ligne 3 et 4
	*/
	
	LUC_Ligne = 0x00;		// cran 2 lignes 16 characteres. poids faible
	OS_Lcd_Send(LUC_Ligne);
	
}

/********************************************************
Fonction: OS_Lcd_Send_noWait
Utilisation: Focntion d'envoi d'une commande  l'cran
Date: 23/11/2003
Auteur: LG
********************************************************/
void OS_Lcd_Send_noWait(uchar LUC_VarSend)
{
	// dclaration de variables locales
	// dbut du corps de la fonction 
	// Envoi de caractere sans tempo
	if (LUC_VarSend & 1)
		LCD_D4 = 1;
	else
		LCD_D4 = 0;
	
	if (LUC_VarSend & 2)
		LCD_D5 = 1;
	else
		LCD_D5 = 0;

	if (LUC_VarSend & 4)
		LCD_D6 = 1;
	else
		LCD_D6 = 0;
	
	if (LUC_VarSend & 8)
		LCD_D7 = 1;
	else
		LCD_D7 = 0;

	Delay1TCY();			//quivalent  1 nop
	LCD_E = 1;				// mise  1 de E ( validation)
	DELAY_600NS;			// delai de 600 ns
	LCD_E = 0;				//mise  0 du bit E
}
/********************************************************
Fonction: OS_Lcd_Send
Utilisation: Fonction d'envoi de commande  l'cran
Date: 23/11/2003
Auteur: LG
********************************************************/
void OS_Lcd_Send(uchar LUC_VarSend)
{									// input LUC_VarSend => contient l octect de command  envoyer.
	OS_Lcd_Send_noWait(LUC_VarSend);
									// envoi d un octets sur l ecran de l afficheur LCD

	DELAY_40US;						// attente de 40us mini pour une commande cf doc
									// envoi des poids faible  l ecran
}

/********************************************************
Fonction: OS_Lcd_Clear
Utilisation: Fonction d'effacement de l'cran
Date: 23/11/2003
Auteur: LG
********************************************************/
void OS_Lcd_Clear(void)
{
	// dclaration de variables locales
	uchar LUC_VarToSend = 0;							
	// dbut du corps de la fonction 

	// cmd=0x01;
	// envoi poids fort
	OS_Lcd_Send(LUC_VarToSend);		// appel de la fonction d envoi de commande
	LUC_VarToSend= 0x01;			// envoi poids faible
	OS_Lcd_Send(LUC_VarToSend);
	DELAY_2MS;						//attente de 2ms
}
/********************************************************
Fonction: OS_Lcd_Char
Utilisation: Fonction d'affichage d'un charactere  l'cran
Date: 23/11/2003
Auteur: LG
********************************************************/
void OS_Lcd_Char(uchar LUC_CharSend)
{									
	// envoi d un charactere  l cran
	// les donnes sont sur les bit RB0- RB3 du port B, le RB4 = E, RB5 = RS
	// dclaration de variables locales
	uchar LUC_VarToSend;					// variable interne  la fonction

	// dbut du corps de la fonction 
	LUC_VarToSend = LUC_CharSend >> 4;		// masquage des poids faible ! 

	LCD_RS = 1;								// RS =1

	OS_Lcd_Send_noWait(LUC_VarToSend);		// envois poid fort

	OS_Lcd_Send_noWait(LUC_CharSend);		// envois poid faible

	LCD_RS = 0;								// RS =0

	DELAY_60US;								// attente de 60s ncessaire pour l'cran, initialement 50s
}
/********************************************************
Fonction: OS_ReturnHome
Utilisation: Fonction de positionnement du curseur en haut  gauche
Date: 23/11/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_ReturnHome(void)
{
	// dclaration de variables locales
	uchar LUC_VarSend;
	// dbut du corps de la fonction 
	LUC_VarSend= 0x00;
	OS_Lcd_Send(LUC_VarSend);
	LUC_VarSend= 0x02;					// poids faible en mode 4 bits
	OS_Lcd_Send(LUC_VarSend);

	DELAY_2MS;
}*/

/********************************************************
Fonction: OS_LcdPrint
Utilisation: Fonction d'affichage d'une chaine de caractere
Date: 23/11/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Printf(const char *LUC_PTR_Chaine)
{

	while(*LUC_PTR_Chaine != '\0')						// Affichage tant qu on est pas  la fin de la chaine
	{ 
		OS_Lcd_Char(*LUC_PTR_Chaine++);
	}
}*/
/********************************************************
Fonction: OS_Lcd_Cur_S_Right
Utilisation: Fonction de dcalage du curseur sur la droite
Date: 23/11/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Cur_S_Right(void)
{
	// dclaration de variables locales
	uchar LUC_VarSend;								// Valeur pour shift droit 0x14
	// dbut du corps de la fonction 
	LUC_VarSend= 0x01;	
	OS_Lcd_Send(LUC_VarSend);
	LUC_VarSend= 0x04;
	OS_Lcd_Send(LUC_VarSend);
}*/
/********************************************************
Fonction: OS_Lcd_Cur_S_Left
Utilisation: Fonction de dcalage du curseur sur la gauche
Date: 23/11/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Cur_S_Left()
{
	// dclaration de variables locales
	uchar LUC_VarSend;								// Valeur pour shift gauche 0x10
	// dbut du corps de la fonction 
	LUC_VarSend= 0x01;	
	OS_Lcd_Send(LUC_VarSend);
	LUC_VarSend= 0x00;
	OS_Lcd_Send(LUC_VarSend);
}*/
/********************************************************
Fonction: OS_Lcd_Line_MoitieGauche
Utilisation: Fonction pour se positionner sur la partie gauche de l'afficheur 1*16
Date: 12/12/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Line_MoitieGauche(void)
{
	// dclaration de variables locales

	// dbut du corps de la fonction 

	OS_Lcd_Send(0x08);
	OS_Lcd_Send(0x00);
}*/
/********************************************************
Fonction: OS_Lcd_Line_MoitieDroite
Utilisation: Fonction pour se positionner sur la partie droite de l'afficheur 1*16
Date: 12/12/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Line_MoitieDroite(void)
{
	// dclaration de variables locales

	// dbut du corps de la fonction 

	OS_Lcd_Send(0x08);
	OS_Lcd_Send(0x00);
}
*/
/********************************************************
Fonction: OS_LcdHexa
Utilisation: Fonction d'affichage de donne au format hexa pour un char
Date: 23/11/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_Hexa(uchar LUC_VarSend)
{
	// affiche  l ecran la valeur Hexa du nombre
	// dclaration de variables locales
	uchar LUC_VarToSend;								// variable temporaire
	// dbut du corps de la fonction 
	OS_Lcd_Char(0x30);								// affichage de 0x...
	OS_Lcd_Char(0x78);
	if ((LUC_VarSend>>4) > 9) 
		LUC_VarToSend= 0x37+(LUC_VarSend>>4);			// envoi quartet poids fort
	else
		LUC_VarToSend= 0x30+(LUC_VarSend>>4);
	OS_Lcd_Char(LUC_VarToSend);
	if ((LUC_VarSend & 0x0F) > 9)
		LUC_VarToSend= 0x37 +(LUC_VarSend & 0x0F);
	else
		LUC_VarToSend= 0x30+( LUC_VarSend  & 0x0F);					// envoi quartet poids faible
	OS_Lcd_Char(LUC_VarToSend);
}*/
/********************************************************
Fonction: OS_LcdHexaUint
Utilisation: Fonction d'affichage de donne au format hexa pour un integer non sign
Date: 05/12/2003
Auteur: LG
********************************************************/
/*
void OS_Lcd_HexaUint(uint LUI_VarSend)
{
	// affiche  l ecran la valeur Hexa du nombre
	// dclaration de variables locales
	uchar LUC_VarToSend;								// variable temporaire
	// dbut du corps de la fonction 
	OS_Lcd_Char(0x30);								// affichage de 0x...
	OS_Lcd_Char(0x78);
	// afficher l'octet de poids fort!
	if (((LUI_VarSend>>8)>>4) > 9) 
		LUC_VarToSend= 0x37+((uchar)(LUI_VarSend>>8)>>4);			// envoi quartet poids fort
	else
		LUC_VarToSend= 0x30+(uchar)((LUI_VarSend>>8)>>4);
	OS_Lcd_Char(LUC_VarToSend);
	
	if (((LUI_VarSend>>8) & 0x0F) > 9)
		LUC_VarToSend= 0x37 +((uchar)(LUI_VarSend>>8) & 0x0F);
	else
		LUC_VarToSend= 0x30+((uchar)(LUI_VarSend>>8) & 0x0F);					// envoi quartet poids faible
	OS_Lcd_Char(LUC_VarToSend);
	
	//affichage de l'octet de poids faible
		if((((uchar)LUI_VarSend) >>4 ) > 9) 
		LUC_VarToSend= 0x37+(((uchar)LUI_VarSend)>>4);			// envoi quartet poids fort
	else
		LUC_VarToSend= 0x30+(((uchar)LUI_VarSend)>>4);
	OS_Lcd_Char(LUC_VarToSend);
	
	if ((((uchar)LUI_VarSend) & 0x0F) > 9)
		LUC_VarToSend= 0x37 +(((uchar)LUI_VarSend) & 0x0F);
	else
		LUC_VarToSend= 0x30+(((uchar)LUI_VarSend)  & 0x0F);					// envoi quartet poids faible
	OS_Lcd_Char(LUC_VarToSend);
	
}
*/


void OS_Lcd_DecimalInt(int _value, char limitSize)
{
	unsigned char tmp;
	unsigned int valueDeci;

	if (_value<0)
	{
		valueDeci = - _value;
		OS_Lcd_Char('-');
	}
	else {
		valueDeci = _value;
		OS_Lcd_Char(' ');
	}

	tmp = 0;
	while (valueDeci>=10000)
	{
		tmp++;
		valueDeci -= 10000;		
	}
	if (limitSize==0 || (limitSize>4))
		OS_Lcd_Char('0'+tmp);
	
	tmp = 0;
	while (valueDeci>=1000)
	{
		tmp++;
		valueDeci -= 1000;		
	}
	if (limitSize==0 || (limitSize>3))
		OS_Lcd_Char('0'+tmp);

	tmp = 0;
	while (valueDeci>=100)
	{
		tmp++;
		valueDeci -= 100;		
	}
	if (limitSize==0 || (limitSize>2))
		OS_Lcd_Char('0'+tmp);

	tmp = 0;
	while (valueDeci>=10)
	{
		tmp++;
		valueDeci -= 10;		
	}
	if (limitSize==0 || (limitSize>1))
		OS_Lcd_Char('0'+tmp);

	tmp = (unsigned char)valueDeci;
	OS_Lcd_Char('0' + tmp);

	OS_Lcd_Char(' ');
}


void OS_Lcd_Distance(unsigned char _value, unsigned char precis)
{
	unsigned char value_deci;
	unsigned char tmp;	
	OS_Lcd_Char(' ');

	value_deci = _value>>1;

	tmp = 0;
	while (value_deci>=100)
	{
		tmp++;
		value_deci -= 100;		
	}
	OS_Lcd_Char('0'+tmp);

	tmp = 0;
	while (value_deci>=10)
	{
		tmp++;
		value_deci -= 10;		
	}
	OS_Lcd_Char('0'+tmp);

	tmp = (unsigned char)value_deci;
	OS_Lcd_Char('0' + tmp);

	if (precis)
	{

		OS_Lcd_Char(',');
		if (_value&1)
			OS_Lcd_Char('5');
		else
			OS_Lcd_Char('0');

		OS_Lcd_Char(' ');

	}


}

void OS_Lcd_PosRobotFloat(float x_p, float y_p, float angle_p)
{
	unsigned char tmp;
	unsigned int value_deci;

	// x
	if (x_p<0) {
		OS_Lcd_Char('-');
		value_deci = -(x_p-0.5);
	}
	else {
		OS_Lcd_Char('+');
		value_deci = x_p+0.5;
	}

	tmp = 0;
	while (value_deci>=100)
	{
		tmp++;
		value_deci -= 100;		
	}
	OS_Lcd_Char('0'+tmp);
	
	tmp = 0;
	while (value_deci>=10)
	{
		tmp++;
		value_deci -= 10;		
	}
	OS_Lcd_Char('0'+tmp);

	tmp = (unsigned char)value_deci;
	OS_Lcd_Char('0' + tmp);

	OS_Lcd_Char(' ');

	// y	
	if (y_p<0) {
		OS_Lcd_Char('-');
		value_deci = -(y_p-0.5);
	}
	else {
		OS_Lcd_Char('+');
		value_deci = y_p+0.5;
	}

	tmp = 0;
	while (value_deci>=100)
	{
		tmp++;
		value_deci -= 100;		
	}
	OS_Lcd_Char('0'+tmp);
	
	tmp = 0;
	while (value_deci>=10)
	{
		tmp++;
		value_deci -= 10;		
	}
	OS_Lcd_Char('0'+tmp);

	tmp = (unsigned char)value_deci;
	OS_Lcd_Char('0' + tmp);

	OS_Lcd_Char(' ');

	
	// angle	
	if (angle_p<0) {
		value_deci = 0.5+(360.0+(angle_p*57.295779)); // *180/PI
	}
	else {
		value_deci = 0.5+(angle_p*57.295779); // *180/PI
	}
	if (value_deci>360)
		value_deci = 360;

	tmp = 0;
	while (value_deci>=100)
	{
		tmp++;
		value_deci -= 100;		
	}
	OS_Lcd_Char('0'+tmp);
	
	tmp = 0;
	while (value_deci>=10)
	{
		tmp++;
		value_deci -= 10;		
	}
	OS_Lcd_Char('0'+tmp);


	tmp = (unsigned char)value_deci;
	OS_Lcd_Char('0' + tmp);

	OS_Lcd_Char(' ');
	OS_Lcd_Char(' ');
	OS_Lcd_Char(' ');


}
