////////////////////////////////////////////////////////////////////////
////                          PicoBot.c                             ////
////                                                                ////
////  Programme principal de commande du robot						////
////                 18F242 quartz 20 MHz ou 10Mhz *4 (PLL) = 40Mhz ////
////  Auteur: 	LG					     							////
////  Date:	 09 novembre 2003                                     	////
////                                                                ////
////////////////////////////////////////////////////////////////////////
////        (C) Copyright 2003 - LG				             		////
////								     							////
////////////////////////////////////////////////////////////////////////

// fichier d'include system du PIC
#include <delays.h>
#include <timers.h>
#include <pwm.h>
#include <adc.h>
#include <usart.h>  
#include <p18f252.h>  // processeur PIC 18F242 

// Define globale du projet
#include "OS_Define.h"



// dclaration des fonctions

// Fonction gnrale pour l'environnement
void OS_ConfigurationProcesseur(void); 	// fonction pour la configuration du processeur
void OS_EnableAllInterrupt(void); // activation des interruptions
void OS_DisableAllInterrupt(void); // dactivation des interruptions
void OS_IntialisationVariableGlobale(void);
void OS_InterruptHigh(void);	// fonction pour la gestion des interruptions de priorit haute
void OS_InterruptLow(void); // fonction pour la gestion des interruptions  de priorit basse

// fonction gnrale de test
void OS_TestMoteur(void); // fonction pour le test des moteurs
void OS_TestAdxL202(void); // fonction pour le test de l'acclrometre

// Fonction grant les interfaces du PIC
void OS_MesureBords(void);	// mesure avec les capteurs sharp pour savoir s'il y a un mur  20cm!
void OS_ClignotementLed400ms(uchar LUC_tempo); // fonction pour faire clignoter une led, multiple de 0.4 s
void OS_ConversionAN(void);						// Fonction pour mesurer les distances sur chaque cot et aussi pour calculer le poids d'une cellule!
uchar OS_EtatBouton(void);

// Fonction pour la commande bas niveau des moteurs
void OS_MoteurAvancer(void);							// fonction pour la gestion des servos moteur 
void OS_MoteurArriere(void);							// fonction pour la gestion des servos moteur
void OS_MoteurDroite(void);							// fonction pour la gestion des servos moteur 
void OS_MoteurGauche(void);							// fonction pour la gestion des servos moteur 
void  OS_MoteurArret(void);							// fonction pour la gestion des servos moteur 


// Fonction pour la mesure des acclration 
void OS_MesureAccelerationStatique(void);				// mesure de l'acclration statique quand on est  l'arret.
void OS_MesureAcceleration(void);

// Fonction pour la commande des dplacements
void OS_Avancer(short long LSL_Distance);
void OS_Reculer(short long LSL_Distance);
void OS_TournerDroite(short long LSL_Distance);
void OS_TournerGauche(short long LSL_Distance);

// Fonction pour la recherche de position dans le labyrinthe
uchar LAB_ReperagePosition(void);
void LAB_RechercheSortie(void);
uchar LAB_RechercheTab(uchar LUC_Mesure,uchar LUC_NumeroRecherche);

// Fonction pour la gestion de l'cran LCD.
void OS_InitLcd(void);							// initialisation de l ecran, Attention au brochage.
void OS_Lcd_Line(uchar ligne);
void OS_Lcd_Send_noWait(uchar LUC_VarSend);				// envoi de commande dans delais
void OS_Lcd_Send(uchar LUC_VarSend);					// envoi de commande avec delais de 50us, minimum pour les ecrans
void OS_Lcd_Hexa(uchar LUC_VarSend);					// affichage d une valeur hexa 0x..
void OS_Lcd_HexaUint(uint LUI_VarSend);					// affichage d une valeur hexa 0x..  partir d'un integer
void OS_Lcd_Binaire(uchar LUC_VarSend);					// Affichage d une valeur Binaire 10101010b
void OS_Lcd_Decimal(uchar LUC_ValSend);					// Affichage d une valeur en decimal (donne d entre sur 1 octet)
void OS_Lcd_Clear(void);						// Effacement de l ecran et position du curseur en 00
void OS_Lcd_Char(uchar LUC_CharSend);					// Affichage du un caractere, passe la valeur Ascii du carac
void OS_Lcd_ReturnHome(void);						// revient en position 00
void OS_Lcd_Printf(uchar *Chaine);					// Affichage d une chaine de caractere
void OS_Lcd_Cur_S_Right();						// decalage du curseur vers la droite
void OS_Lcd_Cur_S_Left();						// decalage du curseur vers la gauche
void OS_Lcd_Line_MoitieGauche(void);			// position sur la partie gauche de l'cran
void OS_Lcd_Line_MoitieDroite(void);				// position sur la partie droite de l'cran.

void calculeCrc(void);

// dclaration de variable extern globale

extern uchar GUC_TAB_Distance[NOMBRE_CAPTEUR_SHARP];
extern uchar GUC_PoidsCellule; // permet de stocker la mesure du poids d'une cellule issue de la fonction OS_ConversionAN
extern uchar GUC_MoteurDroit;	// variable pour la commande de moteur
extern uchar GUC_MoteurGauche;	// variable pour la commande de moteur
extern uint GUI_Timing20ms;		// variable pour la gestion des interruptions.

// variable utile pour le calcule de l'acclration
extern uint GUI_TempsT1X;	// temps pour le crneau X
extern uint GUI_TempsT1Y;	// temps pour le crneau Y
extern uint GUI_TempsTb;	// front descendant du crneaux X
extern uint GUI_TempsTc; 	// front montant du crneaux Y
extern uint GUI_TempsTd;	// front descendant du crneaux Y
extern uint GUI_TempsT2;	// variable pour la priode du signal de l'acclrometre.
extern uchar GUC_EtatAccelerometre; // indique l'etat de l'accelerometre
extern short long GSL_AccelerationX;	// valeur de l'acclration sur l'axe X
extern short long GSL_AccelerationY;	// valeur de l'acclration sur l'axe Y
extern short long GSL_Acceleration ;	// variable pour le calcul de l'acclration moyenne sur un axe!
extern short long GSL_AccelerationStatiqueX;
extern short long GSL_AccelerationStatiqueY;

// variable pour la mesure de distance en X
extern short long GSL_DistanceX ;
extern short long GSL_VitessePrecedenteX ;
extern short long GSL_VitesseX ;
// variable pour la mesure de distance en Y
extern short long GSL_DistanceY ;
extern short long GSL_VitessePrecedenteY ;
extern short long GSL_VitesseY ;

// Variable pour la recherche des positions dans le labyrinthe
extern uchar GUC_Orientation;
extern uchar GUC_PositionY;
extern uchar GUC_PositionX;

// varaible pour indiquer s'il y a un mur  20cm, issue de la mesure OS_MesureBords
extern uchar GUC_Capteur_Avant;	// 
extern uchar GUC_Capteur_Droit;	// 
extern uchar GUC_Capteur_Arriere;	// 
extern uchar GUC_Capteur_Gauche;	// 

// Dfinition du tableau du labyrinthe avec les poids par rapport  la sortie.
rom const uchar GUC_TAB_SortieT[9][9]={
{ 15, 14, 13, 12, 11, 8, 7, 6, 5},
{ 16, 15, 14, 13, 10, 9, 8, 7, 4},
{ 19, 16, 15, 14, 19, 18, 19, 20, 3},
{ 18, 17, 16, 15, 16, 17, 18, 3, 2},
{ 19, 18, 17, 16, 85, 18, 3, 2, 1},
{ 20, 19, 20, 17, 18, 19, 12, 3, 4},
{ 19, 20, 21, 18, 13, 12, 11, 4, 5},
{ 18, 19, 16, 15, 12, 11, 10, 5, 6},
{ 17, 16, 15, 14, 13, 12, 9, 8, 7}
};
// Dfinition du tableau avec le codage du labyrinthe pour la recherche de la positon dans le labyrinthe
uchar const rom GUC_TAB_Tableau[324][5]={
{ 41, 5, 5, 0, 1},
{ 41, 5, 5, 0, 2},
{ 41, 5, 5, 0, 3},
{ 41, 5, 5, 0, 4},
{ 65, 8, 2, 1, 1},
{ 10, 2, 1, 1, 2},
{ 35, 4, 8, 1, 3},
{ 66, 8, 3, 1, 3},
{ 26, 3, 8, 2, 1},
{ 37, 5, 1, 2, 2},
{ 58, 7, 4, 2, 2},
{ 71, 8, 8, 2, 2},
{ 23, 3, 5, 2, 3},
{ 43, 5, 7, 2, 3},
{ 46, 6, 1, 2, 3},
{ 19, 3, 1, 2, 4},
{ 52, 6, 7, 2, 4},
{ 59, 7, 5, 2, 4},
{ 60, 7, 6, 2, 4},
{ 65, 8, 2, 4, 2},
{ 10, 2, 1, 4, 3},
{ 35, 4, 8, 4, 4},
{ 66, 8, 3, 4, 4},
{ 67, 8, 4, 5, 1},
{ 5, 1, 5, 6, 1},
{ 48, 6, 3, 6, 1},
{ 14, 2, 5, 6, 3},
{ 19, 3, 1, 8, 1},
{ 52, 6, 7, 8, 1},
{ 59, 7, 5, 8, 1},
{ 60, 7, 6, 8, 1},
{ 26, 3, 8, 8, 2},
{ 37, 5, 1, 8, 3},
{ 58, 7, 4, 8, 3},
{ 71, 8, 8, 8, 3},
{ 23, 3, 5, 8, 4},
{ 43, 5, 7, 8, 4},
{ 46, 6, 1, 8, 4},
{ 54, 6, 9, 9, 1},
{ 17, 2, 8, 9, 2},
{ 30, 4, 3, 9, 2},
{ 34, 4, 7, 9, 2},
{ 57, 7, 3, 9, 2},
{ 1, 1, 1, 9, 4},
{ 6, 1, 6, 9, 4},
{ 9, 1, 9, 10, 1},
{ 51, 6, 6, 10, 2},
{ 77, 9, 5, 10, 2},
{ 81, 9, 9, 10, 2},
{ 73, 9, 1, 10, 3},
{ 79, 9, 7, 10, 3},
{ 11, 2, 2, 10, 4},
{ 55, 7, 1, 10, 4},
{ 35, 4, 8, 16, 1},
{ 66, 8, 3, 16, 1},
{ 65, 8, 2, 16, 3},
{ 10, 2, 1, 16, 4},
{ 12, 2, 3, 17, 1},
{ 39, 5, 3, 17, 1},
{ 50, 6, 5, 17, 1},
{ 80, 9, 8, 17, 1},
{ 12, 2, 3, 17, 3},
{ 39, 5, 3, 17, 3},
{ 50, 6, 5, 17, 3},
{ 80, 9, 8, 17, 3},
{ 16, 2, 7, 18, 1},
{ 42, 5, 6, 18, 2},
{ 72, 8, 9, 18, 2},
{ 2, 1, 2, 18, 3},
{ 7, 1, 7, 18, 3},
{ 32, 4, 5, 18, 3},
{ 74, 9, 2, 18, 3},
{ 18, 2, 9, 18, 4},
{ 61, 7, 7, 18, 4},
{ 67, 8, 4, 20, 2},
{ 21, 3, 3, 21, 1},
{ 64, 8, 1, 21, 4},
{ 8, 1, 8, 22, 1},
{ 25, 3, 7, 22, 1},
{ 36, 4, 9, 22, 2},
{ 15, 2, 6, 22, 3},
{ 53, 6, 8, 22, 4},
{ 5, 1, 5, 24, 2},
{ 48, 6, 3, 24, 2},
{ 14, 2, 5, 24, 4},
{ 56, 7, 2, 25, 3},
{ 28, 4, 1, 25, 4},
{ 68, 8, 5, 25, 4},
{ 4, 1, 4, 26, 1},
{ 70, 8, 7, 26, 2},
{ 20, 3, 2, 26, 4},
{ 23, 3, 5, 32, 1},
{ 43, 5, 7, 32, 1},
{ 46, 6, 1, 32, 1},
{ 19, 3, 1, 32, 2},
{ 52, 6, 7, 32, 2},
{ 59, 7, 5, 32, 2},
{ 60, 7, 6, 32, 2},
{ 26, 3, 8, 32, 3},
{ 37, 5, 1, 32, 4},
{ 58, 7, 4, 32, 4},
{ 71, 8, 8, 32, 4},
{ 2, 1, 2, 33, 1},
{ 7, 1, 7, 33, 1},
{ 32, 4, 5, 33, 1},
{ 74, 9, 2, 33, 1},
{ 18, 2, 9, 33, 2},
{ 61, 7, 7, 33, 2},
{ 16, 2, 7, 33, 3},
{ 42, 5, 6, 33, 4},
{ 72, 8, 9, 33, 4},
{ 3, 1, 3, 34, 1},
{ 75, 9, 3, 34, 1},
{ 27, 3, 9, 34, 2},
{ 78, 9, 6, 34, 2},
{ 3, 1, 3, 34, 3},
{ 75, 9, 3, 34, 3},
{ 27, 3, 9, 34, 4},
{ 78, 9, 6, 34, 4},
{ 1, 1, 1, 36, 1},
{ 6, 1, 6, 36, 1},
{ 54, 6, 9, 36, 2},
{ 17, 2, 8, 36, 3},
{ 30, 4, 3, 36, 3},
{ 34, 4, 7, 36, 3},
{ 57, 7, 3, 36, 3},
{ 63, 7, 9, 37, 2},
{ 76, 9, 4, 37, 3},
{ 62, 7, 8, 37, 4},
{ 11, 2, 2, 40, 1},
{ 55, 7, 1, 40, 1},
{ 9, 1, 9, 40, 2},
{ 51, 6, 6, 40, 3},
{ 77, 9, 5, 40, 3},
{ 81, 9, 9, 40, 3},
{ 73, 9, 1, 40, 4},
{ 79, 9, 7, 40, 4},
{ 24, 3, 6, 41, 1},
{ 44, 5, 8, 41, 1},
{ 13, 2, 4, 41, 2},
{ 49, 6, 4, 41, 4},
{ 22, 3, 4, 42, 2},
{ 40, 5, 4, 42, 2},
{ 45, 5, 9, 42, 3},
{ 31, 4, 4, 42, 4},
{ 38, 5, 2, 42, 4},
{ 10, 2, 1, 64, 1},
{ 35, 4, 8, 64, 2},
{ 66, 8, 3, 64, 2},
{ 65, 8, 2, 64, 4},
{ 67, 8, 4, 65, 4},
{ 17, 2, 8, 66, 1},
{ 30, 4, 3, 66, 1},
{ 34, 4, 7, 66, 1},
{ 57, 7, 3, 66, 1},
{ 1, 1, 1, 66, 3},
{ 6, 1, 6, 66, 3},
{ 54, 6, 9, 66, 4},
{ 12, 2, 3, 68, 2},
{ 39, 5, 3, 68, 2},
{ 50, 6, 5, 68, 2},
{ 80, 9, 8, 68, 2},
{ 12, 2, 3, 68, 4},
{ 39, 5, 3, 68, 4},
{ 50, 6, 5, 68, 4},
{ 80, 9, 8, 68, 4},
{ 64, 8, 1, 69, 3},
{ 21, 3, 3, 69, 4},
{ 56, 7, 2, 70, 2},
{ 28, 4, 1, 70, 3},
{ 68, 8, 5, 70, 3},
{ 18, 2, 9, 72, 1},
{ 61, 7, 7, 72, 1},
{ 16, 2, 7, 72, 2},
{ 42, 5, 6, 72, 3},
{ 72, 8, 9, 72, 3},
{ 2, 1, 2, 72, 4},
{ 7, 1, 7, 72, 4},
{ 32, 4, 5, 72, 4},
{ 74, 9, 2, 72, 4},
{ 63, 7, 9, 73, 1},
{ 76, 9, 4, 73, 2},
{ 62, 7, 8, 73, 3},
{ 13, 2, 4, 74, 1},
{ 49, 6, 4, 74, 3},
{ 24, 3, 6, 74, 4},
{ 44, 5, 8, 74, 4},
{ 67, 8, 4, 80, 3},
{ 64, 8, 1, 81, 2},
{ 21, 3, 3, 81, 3},
{ 76, 9, 4, 82, 1},
{ 62, 7, 8, 82, 2},
{ 63, 7, 9, 82, 4},
{ 64, 8, 1, 84, 1},
{ 21, 3, 3, 84, 2},
{ 47, 6, 2, 86, 2},
{ 69, 8, 6, 86, 4},
{ 53, 6, 8, 88, 1},
{ 8, 1, 8, 88, 2},
{ 25, 3, 7, 88, 2},
{ 36, 4, 9, 88, 3},
{ 15, 2, 6, 88, 4},
{ 69, 8, 6, 89, 1},
{ 47, 6, 2, 89, 3},
{ 33, 4, 6, 90, 1},
{ 14, 2, 5, 96, 1},
{ 5, 1, 5, 96, 3},
{ 48, 6, 3, 96, 3},
{ 15, 2, 6, 97, 1},
{ 53, 6, 8, 97, 2},
{ 8, 1, 8, 97, 3},
{ 25, 3, 7, 97, 3},
{ 36, 4, 9, 97, 4},
{ 28, 4, 1, 100, 1},
{ 68, 8, 5, 100, 1},
{ 56, 7, 2, 100, 4},
{ 69, 8, 6, 101, 2},
{ 47, 6, 2, 101, 4},
{ 29, 4, 2, 102, 2},
{ 29, 4, 2, 102, 4},
{ 20, 3, 2, 104, 1},
{ 4, 1, 4, 104, 2},
{ 70, 8, 7, 104, 3},
{ 33, 4, 6, 105, 2},
{ 37, 5, 1, 128, 1},
{ 58, 7, 4, 128, 1},
{ 71, 8, 8, 128, 1},
{ 23, 3, 5, 128, 2},
{ 43, 5, 7, 128, 2},
{ 46, 6, 1, 128, 2},
{ 19, 3, 1, 128, 3},
{ 52, 6, 7, 128, 3},
{ 59, 7, 5, 128, 3},
{ 60, 7, 6, 128, 3},
{ 26, 3, 8, 128, 4},
{ 14, 2, 5, 129, 2},
{ 5, 1, 5, 129, 4},
{ 48, 6, 3, 129, 4},
{ 51, 6, 6, 130, 1},
{ 77, 9, 5, 130, 1},
{ 81, 9, 9, 130, 1},
{ 73, 9, 1, 130, 2},
{ 79, 9, 7, 130, 2},
{ 11, 2, 2, 130, 3},
{ 55, 7, 1, 130, 3},
{ 9, 1, 9, 130, 4},
{ 42, 5, 6, 132, 1},
{ 72, 8, 9, 132, 1},
{ 2, 1, 2, 132, 2},
{ 7, 1, 7, 132, 2},
{ 32, 4, 5, 132, 2},
{ 74, 9, 2, 132, 2},
{ 18, 2, 9, 132, 3},
{ 61, 7, 7, 132, 3},
{ 16, 2, 7, 132, 4},
{ 36, 4, 9, 133, 1},
{ 15, 2, 6, 133, 2},
{ 53, 6, 8, 133, 3},
{ 8, 1, 8, 133, 4},
{ 25, 3, 7, 133, 4},
{ 70, 8, 7, 134, 1},
{ 20, 3, 2, 134, 3},
{ 4, 1, 4, 134, 4},
{ 27, 3, 9, 136, 1},
{ 78, 9, 6, 136, 1},
{ 3, 1, 3, 136, 2},
{ 75, 9, 3, 136, 2},
{ 27, 3, 9, 136, 3},
{ 78, 9, 6, 136, 3},
{ 3, 1, 3, 136, 4},
{ 75, 9, 3, 136, 4},
{ 22, 3, 4, 138, 1},
{ 40, 5, 4, 138, 1},
{ 45, 5, 9, 138, 2},
{ 31, 4, 4, 138, 3},
{ 38, 5, 2, 138, 3},
{ 1, 1, 1, 144, 2},
{ 6, 1, 6, 144, 2},
{ 54, 6, 9, 144, 3},
{ 17, 2, 8, 144, 4},
{ 30, 4, 3, 144, 4},
{ 34, 4, 7, 144, 4},
{ 57, 7, 3, 144, 4},
{ 56, 7, 2, 145, 1},
{ 28, 4, 1, 145, 2},
{ 68, 8, 5, 145, 2},
{ 49, 6, 4, 146, 2},
{ 24, 3, 6, 146, 3},
{ 44, 5, 8, 146, 3},
{ 13, 2, 4, 146, 4},
{ 62, 7, 8, 148, 1},
{ 63, 7, 9, 148, 3},
{ 76, 9, 4, 148, 4},
{ 47, 6, 2, 149, 1},
{ 69, 8, 6, 149, 3},
{ 33, 4, 6, 150, 4},
{ 29, 4, 2, 153, 1},
{ 29, 4, 2, 153, 3},
{ 73, 9, 1, 160, 1},
{ 79, 9, 7, 160, 1},
{ 11, 2, 2, 160, 2},
{ 55, 7, 1, 160, 2},
{ 9, 1, 9, 160, 3},
{ 51, 6, 6, 160, 4},
{ 77, 9, 5, 160, 4},
{ 81, 9, 9, 160, 4},
{ 20, 3, 2, 161, 2},
{ 4, 1, 4, 161, 3},
{ 70, 8, 7, 161, 4},
{ 45, 5, 9, 162, 1},
{ 31, 4, 4, 162, 2},
{ 38, 5, 2, 162, 2},
{ 22, 3, 4, 162, 4},
{ 40, 5, 4, 162, 4},
{ 49, 6, 4, 164, 1},
{ 24, 3, 6, 164, 2},
{ 44, 5, 8, 164, 2},
{ 13, 2, 4, 164, 3},
{ 33, 4, 6, 165, 3},
{ 31, 4, 4, 168, 1},
{ 38, 5, 2, 168, 1},
{ 22, 3, 4, 168, 3},
{ 40, 5, 4, 168, 3},
{ 45, 5, 9, 168, 4}
};


